%
% Get the circle in the image that has a radius in the given
% range, and whose support (number of edge elements agreeing with hypothesis)
% is at least a certain percentage from its circumference.
%
% NOTE: matlab assumes isotropic pixels, no notion of spacing as is common
%       in medical imaging.
%
% @param inputImage The image that contains the circles.
%
% @param radiusRange A 2x1 or 1x2 vector. The sphere estimate is valid iff 
%                    the radius is inside the user defined range 
%                    [min(radiusRange), max(radiusRange)].
%
% @param minimalCircumferencePercentage A circle hypothesis is accepted if 
%                                       its support (number of pixels that 
%                                       are on the circle) is larger than 
%                                       minimalCircumferencePercentage*2*pi*r, 
%                                       r is in pixels.
%
% @author Ziv Yaniv (zivy@isis.georgetown.edu)
%
function circleParameters = localize2DFiducialsUsingRansac(inputImage, radiusRange, minimalCircumferencePercentage)

  circleParameters = [];
  
          %perform edge detection using the Canny detector, 
          %NOTE: the only reason we can use matlab's 'edge' function is
          %that the pixels are isotropic 0.3730^2 (see dicom header)
          %in general image processing in matlab does not have the notion
          %of pixel/voxel spacing, everything is assumed to be isotropic
  
  %        
  % NOTE: There is a bug in the MATLAB implementation of the canny edge 
  %       detector. For a uniform image instead of returning all 0, 
  %       all locations are 1 except the border which is 0.
  %       I'm still using it as this only happens for a few images and these
  %       outlying localizations are implicitly ignored using RANSAC.
  %
  std = 2.0;
  edgeImage = edge(inputImage, 'canny', [], std);
  
       %get all the edge points (marked by 1)
  curPointIndexes = find(edgeImage == 1);
  [y,x] = ind2sub(size(edgeImage), curPointIndexes);
  curPointSet = [x';y'];
    
           %a point is on the circle if it is less than 'agreementTolerance' pixels
           %away
  CIRCLE_AGREEMENT_TOLERANCE = 1.0; 
  [curCircleParameters, curConsensusSetIndexes] = ransacSphereConstrainedRadius(curPointSet, radiusRange,...
                                                                                CIRCLE_AGREEMENT_TOLERANCE);  
           %constant that multiplies the circle radius to get the minimal
           %number of pixels that we are willing to accept as representing
           %a valid circle
  c = 2*pi*minimalCircumferencePercentage;
  if ~isempty(curCircleParameters)
    if size(curConsensusSetIndexes,2)>=c*curCircleParameters(3)
      circleParameters = curCircleParameters;
    end
  end
