%
% This function is an implementation of the RANdom SAmple Consensus (RANSAC)
% framework. It is based on the original paper by Fischler and Bolles and
% its description in the book by Hartely and Zisserman.
%
% Fischler M.A., Bolles R.C., "Random Sample Consensus: A Paradigm for 
% Model Fitting with Applications to Image Analysis and Automated 
% Cartography", Communications of the ACM, Vol. 24(6), 1981.
% 
% Hartely R., Zisserman A., "Multiple View Geometry in Computer Vision"
%
% @param data An array containing the point data required for estimation, each column is a data point.
%
% @param numForExstimate The minimal number of data elements required for
%                        an exact estimate (e.g. three points for estimating 
%                        a circle).
%
% @param exactEstimation A handle to a function that is invoked with a cell 
%                        array with the minimal number of elements. Returns a  
%                        vector containing the parameters of the exact estimate
%                        of the specific parameteric entity. If the data
%                        elements are in a degenerate configuration then
%                        the vector is of length zero.
%
% @param agreeWithModel A handle to a function that receives a model
%                       (parameter vector) and the data elements cell array
%                       and returns the indexes of the data elements that
%                       agree with the specific model (e.g. when fitting a
%                       circle, all the indexes of the points whose
%                       distance to the specified circle is less than some 
%                       threshold 't'.
%
% @param desiredProbabilityForNoOutliers The probability that at least one 
%                                        of the selected subsets doesn't 
%                                        contain an outlier, must be in 
%                                        (0,1).
%
% @param varargin{1} Optional parameter. A handle to a function that is given
%                                     a cell array of elements and computes a 
%                                     least squares estimate of the parametric 
%                                     entity we are fitting. 
%
% @return consensusSet The largest consensus set obtained.
% 
% @return varargout{1} Optional return value. If a least squares function 
%                      handle was given in the input (varargin{1}) then the 
%                      parameters estimate will be returned in this
%                      variable.
%
% @author Ziv Yaniv (zivy@isis.georgetown.edu)
%
function [consensusSet,varargout] = ransac(data,...
                                           numForEstimate,...
                                           exactEstimation,... 
                                           agreeWithModel,... 
                                           desiredProbabilityForNoOutliers,...
                                           varargin)
  
  consensusSet = [];
  numDataObjects = size(data,2);
              %there are less data objects than the minimum required for 
              %an exact fit, or the given probability is not in (0,1)
  if numDataObjects < numForEstimate ||...
     desiredProbabilityForNoOutliers>=1.0 ||... 
     desiredProbabilityForNoOutliers<=0.0
           %user requested the least squares estimate
     if nargin == 6 && nargout == 2
       varargout{1} = [];
     end
    return;
  end

  numerator = log(1.0-desiredProbabilityForNoOutliers);
	allTries = nchoosek(numDataObjects, numForEstimate);
  
             %initalize with 0 so that the first computation which gives
             %any type of fit will be set to best
  numVotesForBest = 0; 
             %initialize with the number of all possible subsets
	numTries = allTries; 
	i=1;
  while i<=numTries
				  %randomly select data for exact model fit 
          %('numForEstimate' objects). Do this by drawing elements with
          %replacement using a uniform distribution. As the elements are
          %drawn with replacement and we require 'numForEstimate' unique
          %elements we repeat till the subset consists of unique elements
    curSubsetIndexes = ceil(rand(1, numForEstimate)*numDataObjects);
    while size(curSubsetIndexes,2) ~= size(unique(curSubsetIndexes),2)
      curSubsetIndexes = ceil(rand(1, numForEstimate)*numDataObjects);
    end
        
    exactEstimateParameters = exactEstimation(data(:,curSubsetIndexes));
             %the subset we chose was a degenerate configuration    
    if isempty(exactEstimateParameters)
      i = i+1;
      continue;
    end
             %non-degenerate configuration, we have an estimate, see how many
             %data elements agree with this estimate    
    curConsensusSet = agreeWithModel(exactEstimateParameters, data);
    numVotesForCur = size(curConsensusSet,2);
             %found a larger consensus set     
    if numVotesForCur > numVotesForBest
      numVotesForBest = numVotesForCur;
      consensusSet = curConsensusSet;
              %all data objects agree on this model, early termination
      if numVotesForBest == numDataObjects
        i=numTries;
      else
        denominator = log(1 - (numVotesForCur/numDataObjects)^numForEstimate);
					            %there are cases when the probablistic number of tries is 
                      %greater than all possible sub-sets
        numTries = min(allTries, ceil(numerator/denominator));
      end
    end
    i = i+1;
  end    
  	             %if the user provided a least squares estimation method
                 %compute the least squares estimate using the largest sub set
                 %make sure that it exists (e.g. n>3 instances of the same point
                 %are possible but they cannot yield an estimate for a circle)
  if nargin == 6 && nargout == 2                 
    if numVotesForBest > 0
      varargout{1} = varargin{1}(data(:,consensusSet));
    else
      varargout{1} = [];
    end
  end
    
  
