%
% This function estimates the intersection point of the given set
% of rays, robust ray intersection using the RANSAC algorithm.
% 
% @param rays A 6xn matrix representing the set of rays [p;n].
%             The first three rows, p, are the coordinates of the 3D point 
%             from which the ray emenates and the remaining three rows,
%             n, are the ray's direction (||n||=1).
%
% @param agreementTolerance A ray is in agreement with an intersection point 
%                           hypothesis if the point-ray distance is less 
%                           than 'agreementTolerance'.
% 
% @return consensusSet Indexes of the largest consensus set obtained using 
%                      the RANSAC framework. 
%                           
% @author Ziv Yaniv (zivy@isis.georgetown.edu)
%
function [intersectionPoint, consensusSet] = ransacRayIntersections(rays,...
                                                                    agreementTolerance)

               %two rays uniquely define an intersection point (valid if
               %they intersect and are not skew or parallel).
numForExactEstimate = 2;

                %use anonymous functions to pass additional parameters for
                %the consensus set voting
agreeWithModel = @(intersectionPoint, rays) raysAgreeWithModel(intersectionPoint, rays, agreementTolerance);

desiredProbabilityForNoOutliers = 0.99;

[consensusSet, intersectionPoint] = ransac(rays,...
                                           numForExactEstimate,...
                                           @exactRayIntersection,...
                                           agreeWithModel,...
                                           desiredProbabilityForNoOutliers, ...
                                           @leastSquaresRayIntersection);
                                       
%
% This function computes the intersection point of two rays. If the rays
% are skew the returned point minimizes the sum of squared distances
% to the rays. Based on the code found in Graphics Gems p.304 "Intersection 
% of Two Lines in Three-Space", Ronald Goldman. 
%
% @param rays A 6x2 array where each column is a ray [p;n], ||n||=1.
%
% @return pnt A 3x1 vector [x; y; z] if the rays intersect or are skew, 
%                otherwise an empty vector.
%
function  pnt = exactRayIntersection(rays)

       %this is where I make the assumption that ||n1||=||n2||=1 as the 
       %threshold is 
       %||cross(n1,n2)||^2 = (||n1||^2*||n2||^2sin(minimalAngularDeviation)^2 = 
       %                     sin(minimalAngularDeviation)^2 = CROSS_EPS
       %with minimalAngularDeviation given as 1 degree
  CROSS_EPS = sin(pi/180.0)^2;     
  
  pnt = [];
                %we require two rays
  if size(rays,2) < 2
    return;
  end
 
  p21 = rays(1:3,2) - rays(1:3,1);
  n1Crossn2 = cross(rays(4:6,1), rays(4:6,2));
  denominator = n1Crossn2'*n1Crossn2;	
       %rays are parallel
  if(denominator<CROSS_EPS)
    return;
  end
         %rays are not parallel (skew or intersecting)
  t1 = (cross(p21,rays(4:6,2))'*n1Crossn2)/denominator;
  t2 = (cross(p21,rays(4:6,1))'*n1Crossn2)/denominator;
              %the lines {t in (-inf, inf)} intersect, but the rays 
              %{t in [0, inf)} don't
	if(t1<0 || t2<0)
		return;
  end
                 %the intersection is the mid-point
  pnt = (rays(1:3,1) + t1*rays(4:6,1) +... 
         rays(1:3,2) + t2*rays(4:6,2))/2.0; 
  
    
                
                
%
% This function gets the consensus set for the given parametrization. This is the
% set of all rays that are closer than 'agreementTolerance' to the
% intersection point.
%
% @param intersectionPoint A 3x1 vector [x; y; z], defining a specific 
%                         intersection point.
%
% @param rays A 6xn array, where each column is a 3D ray [p;n] where ||n||=1.
%              
% @param agreementTolerance A ray, r, agrees with the hypothesized intersection, p, if
%                           the distance between the point and ray is less
%                           than agreementTolerance.
%
function curConsensusSet = raysAgreeWithModel(intersectionPoint, rays, agreementTolerance)

  dirs = intersectionPoint(:,ones(1,size(rays,2))) - rays(1:3,:);
  perps = cross(dirs,rays(4:6,:),1);    
  distances = sqrt(sum(perps.^2,1));  
  curConsensusSet = find(distances < agreementTolerance);
  
%
% Given multiple rays compute the point 'x' that minimizes the sum of squared
% distances to all rays. If the rays intersect this will be their
% intersection point.
% A ray is defined as:
%         r(t) = p + t*n , 
%         where |n| = 1 and t>= 0 .
%
% The minimization is thus given by: 
%               x = min(sum(||(x-p_i) - <n_i,(x-p_i)>n_i||^2))
%
% @param rays A 6xn matrix representing the set of rays [p;n].
%             The first three rows, p, are the coordinates of the 3D point 
%             from which the ray emenates and the remaining three rows,
%             n, are the ray's direction (||n||=1).
% 
% @return pnt The point which minimizes the distance to all rays. If they
%             intersect, this is their intersection point.
% 
function pnt = leastSquaresRayIntersection(rays)

  n = size(rays,2);

  b = [];
  tmp = sum(rays(4:6,:) .* rays(1:3,:));
  b = tmp(ones(3,1),:) .* rays(4:6,:);
  b = (sum(rays(1:3,:)'))' - (sum(b'))';

  A = zeros(3,3);
  for i = 1:n
    A = A + (rays(4:6,i) * rays(4:6,i)');
  end
  A = -A;
  A(1,1) = A(1,1) + n;
  A(2,2) = A(2,2) + n;
  A(3,3) = A(3,3) + n;
  
  if(rank(A) ~= 3)
    error('Rays are parallel');
  end

  pnt = pinv(A)*b;
