%
% This function estimates the hypersphere (m-D sphere) parameters for the 
% given point data set using the RANSAC robust estimation framework. The
% estimated radius is constrained to be in [minRadius, maxRadius].
% 
% @param points A mxn matrix representing the m-D point coordinates.
%
% @param radiusRange The expected radius range [minRadius, maxRadius].
%
% @param agreementTolerance A point is in agreement with a hypersphere 
%                           hypothesis if its distance from the sphere is 
%                           less than 'agreementTolerance'.
% 
% @return consensusSet Indexes of the largest consensus set obtained using 
%                      the RANSAC framework. 
%
% @return sphereParameters The sphere parameters [x_1,...x_m,r], sphere
%                          center and radius.
% @author Ziv Yaniv (zivy@isis.georgetown.edu)
%
function [sphereParameters, consensusSet] = ransacSphereConstrainedRadius(dataPoints, radiusRange,...
                                                                          agreementTolerance)

               %m+1 points in a general configuration uniquely define a 
               %m-D sphere
numForExactEstimate = size(dataPoints,1) + 1;

                %use anonymous functions to pass additional function 
                %parameters for
exactEstimation = @(points) exactSphereEstimate(points, radiusRange);
agreeWithModel = @(sphereParameters, points) pointsAgreeWithModel(sphereParameters, points, agreementTolerance);
leastSquaresEstimate = @(points) leastSquaresSphereEstimate(points, radiusRange);

desiredProbabilityForNoOutliers = 0.99;

[consensusSet, sphereParameters] = ransac(dataPoints,...
                                          numForExactEstimate,...
                                          exactEstimation,...
                                          agreeWithModel,...
                                          desiredProbabilityForNoOutliers, ...
                                          leastSquaresEstimate);

%                                       
% This function computes the m-D sphere defined by m+1 points in a general
% configuration. The sphere is parametrized by its center and radius 
% [c=(c_1;c_2;...;c_m); r]. 
% We know that all the points are on the sphere, that is:
% (p_1-center)^2 = (p_2-center)^2 =...= (p_{m+1}-center)^2 = r^2 
% This gives us m linearly independent equations in m unknowns (c_1,c_2,...,c_m):
%   (p_1-c)^2 = (p_2-c)^2
%   (p_1-c)^2 = (p_3-c)^2
%             :
%   (p_1-c)^2 = (p_{m+1}-c)^2, 
% after simplification we get (Ax=b):
%   [(p_11 - p_21)       (p_12 - p_22)     ...  (p_1m - p_2m)    ] [center_1]         [(p_11^2 - p_21^2)     +   (p_12^2 - p_22^2)     + ... + (p_1m^2 - p_2m^2)    ]
%   [(p_11 - p_31)       (p_12 - p_32)     ...  (p_1m - p_3m)    ] [center_2]  = 0.5* [(p_11^2 - p_31^2)     +   (p_12^2 - p_32^2)     + ... + (p_1m^2 - p_3m^2)    ]
%   [      :                   :            :         :          ] [   :    ]         [        :                         :                 :           :            ]
%   [(p_11 - p_{m+1}1)   (p_12 - p_{m+1}2) ...  (p_1m - p_{m+1}m)] [center_m]         [(p_11^2 - p_{m+1}1^2) +  (p_12^2 - p_{m+1}2^2)  + ... + (p_1m^2 - p_{m+1}m^2)]
%
% Solving this set of linear equations yields the (hyper)sphere center, 
% after which the radius is computed as sqrt((center-p_1)^T(center-p_1)).                                          
%
% If the matrix is singular (collinear points) the result is empty,
% otherwise result = [c_1; c_2;...; c_m; r].
%
% @param points A m x (m+1) array where each column is a m-D point.
%
% @param radiusRange A 2x1 or 1x2 vector. The sphere estimate is valid iff 
%                    the radius is inside the user defined range 
%                    [min(radiusRange), max(radiusRange)].
%
% @return sphereParameters A (m+1) x 1 vector [c_1; c_2;...; c_m; r] if the points 
%                          define a valid sphere, otherwise an empty vector.
%
function  sphereParameters = exactSphereEstimate(points, radiusRange)

sphereParameters = [];

         %point dimensionality
m = size(points,1);

       %create the matrix A
p = points(:,1);
A = p(:,ones(1,m)) - points(:,2:end);
A = A';
       %create the vector b 
tmp = sum(points.^2);
p1NormSquared = tmp(1);
b = 0.5* (p1NormSquared(ones(m,1)) - tmp(2:end)');

           %check that the m+1 points are in a general configuration
if rank(A) ~= m
  return;
end
       
sphereParameters = A\b;
sphereParameters(m+1) = sqrt((points(:,1) - sphereParameters)'*(points(:,1) - sphereParameters));  
           %estimated radius has to be with the specified ranges
if sphereParameters(m+1) > max(radiusRange) || sphereParameters(m+1) < min(radiusRange)
  sphereParameters = [];
end

  
%
% This function gets the consensus set for the given parametrization. This is the
% set of all points that are closer than 'agreementTolerance' to the
% sphere.
%
% @param sphereParameters A (m+1) x 1 vector [c_1; c_2;...; c_m; r], 
%                         defining a specific sphere.
%
% @param points A m x n array, where each column is a m-D point.
%              
% @param agreementTolerance A point, p, agrees with the hypothesized sphere, [c,r], if
%                           |sqrt((p-c)'(p-c)) - r| < agreementTolerance
%
function curConsensusSet = pointsAgreeWithModel(sphereParameters, points, agreementTolerance)

m = size(points,1);
sphereCenter = sphereParameters(1:m);
distances = abs(sqrt(sum((points - sphereCenter(:,ones(1,size(points,2)))).^2, 1)) - sphereParameters(m+1));
curConsensusSet = find(distances < agreementTolerance);
  
%
% This function computes the algebraic least squares sphere estimate.
%
% @param points A m x n array, where each column is a m-D point.
%
% @param radiusRange A 2x1 or 1x2 vector. The sphere estimate is valid iff 
%                    the radius is inside the user defined range 
%                    [min(radiusRange), max(radiusRange)].
%
% @return sphereParameters A (m+1) x 1 vector. The sphere parametrization 
%                          [c_1; c_2;...; c_m; r], or an empty vector if 
%                          the solution estimated by the algebraic method 
%                          is not a real sphere.
%
function sphereParameters = leastSquaresSphereEstimate(points, radiusRange)

sphereParameters = [];  
m = size(points,1);
  
A = [-2*points', ones(size(points,2),1)];
        %check if the points are all collinear
if rank(A) < m+1
  return;
end
    
b = -sum(points.^2)';
x = A\b;
rSquared = x(1:m)'*x(1:m) - x(m+1);
  
        %check if the solution we have is a real sphere  
if rSquared < 0
  return;
end
sphereParameters = [x(1:m); sqrt(rSquared)];
           %estimated radius has to be with the specified ranges
if sphereParameters(m+1) > max(radiusRange) || sphereParameters(m+1) < min(radiusRange)
  sphereParameters = [];
end
  
  
