%
% This function reads a single frame from a cine loop. 
% This function is valid for the current (Jan. 2010) data acquired with the
% Siemens Axiom Artis dFA system and transfered via burning a CD using the
% Siemens software from the acquisition computer.
%
% @param info The dicom info structure of the file that should be read.
%
% @param i The frame number we want to read. Valid values are 
%          in [1,info.NumberOfFrames]
%
% @author Ziv Yaniv (zivy@isis.georgetown.edu)
%
function img = readFramePadded(info, i)

  if i>info.NumberOfFrames || i<1
    error('Requested frame (%d) out of range of valid frames (1-%d).',...
          i, info.NumberOfFrames);
  end

       %when we get the data from the siemens computer by performing DICOM 
       %transfer it has the original size which is compatible with the
       %calibration matrices, these values are found in private Siemens
       %fields
  DICOM_TRANSFER_COLUMNS = double(info.Private_0021_1023);
  DICOM_TRANSFER_ROWS = double(info.Private_0021_1022);

     %when we get the data from the siemens computer by burnning a CD it 
     %is scaled and we need to undo this scaling so that the images are 
     %compatible with the calibration matrices
  CD_TRANSFER_COLUMNS = double(info.Columns);
  CD_TRANSFER_ROWS = double(info.Rows);

     %read the image using the matlab function if the frames are compressed, 
     %otherwise read the raw data. This is a workaround due to issues with
     %the matlab reading function for the larger uncompressed cine loops
  
     %The DICOM tag that indiciates file compression is 
     % 0002:0010 (TransferSyntaxUID) defined in DICOM standard part 5
     % avaliable from ftp://medical.nema.org/medical/dicom/2000/draft/00_05dr.pdf
     %
     % The following table was copied from, http://www.sph.sc.edu/comd/rorden/dicom.html
     %The relevant values are:
     %Transfer Syntax UID  	 |  Definition
     %-----------------------|--------------------------------------
     %1.2.840.10008.1.2 	   | Raw data, Implicit VR, Little Endian
     %1.2.840.10008.1.2.x  	 | Raw data, Eplicit VR, x = 1: Little Endian, x = 2: Big Endian
     %1.2.840.10008.1.2.4.xx | JPEG compression xx = 50-64: Lossy JPEG
     %                       | xx = 65-70: Lossless JPEG
     %1.2.840.10008.1.2.5 	 | Lossless Run Length Encoding
  
  if strcmp(info.TransferSyntaxUID, '1.2.840.10008.1.2.1')
    fileName = info.Filename;
    frameSizeInBytes = CD_TRANSFER_ROWS*CD_TRANSFER_COLUMNS*(double(info.BitsAllocated)/8);
                     %compute the location where pixel data starts, this is a
                     %hack because of problems with matlab reading
                     %uncompressed cine loops
    fileInfo = dir(fileName);
    fileSize=fileInfo.bytes;
    offset = fileSize - frameSizeInBytes*info.NumberOfFrames;  
    fid = fopen(fileName);
    if fid == -1
      error('Failed to read frame (%d), file (%s) cannot be accessed.',...
            i, fileName);
    end  
           %skip from the begining of the file to the frame's start
           %location
    fseek(fid, offset + (i-1)*frameSizeInBytes, 'bof');
    img = fread(fid,[double(info.Columns), double(info.Rows)],'int16');
       %we need to rotate the image by 270^o so that it matches the
       %projection matrices
    img = rot90(img,3);
    fclose(fid);
  else 
    try
           %read the requested frame
    img = dicomread(info, 'frames', i);
                 %we need to flip the image so that it matches the projection 
                 %matrices    
    img = fliplr(img);
    catch
      error('Failed to read frame (%d), file (%s) cannot be accessed.',...
             i, info.Filename);    
    end
  end
           %remove the rows of black pixels that pad the image (dataOffset)
           %and resample this subimage to the correct size as defined by
           %the DICOM transfer sizes. This process undoes the scaling
           %performed on the original image. First it is scaled to the
           %desired width, then the remaining height is padded so that the
           %width and height are equal.
  dataOffset = (CD_TRANSFER_ROWS - ((CD_TRANSFER_COLUMNS/DICOM_TRANSFER_COLUMNS)*DICOM_TRANSFER_ROWS))/2;
  sampleSpacingY = (CD_TRANSFER_ROWS - 2*dataOffset)/(DICOM_TRANSFER_ROWS-1);
  sampleSpacingX = (CD_TRANSFER_COLUMNS-1)/(DICOM_TRANSFER_COLUMNS-1);
  [newX, newY] = meshgrid([1:sampleSpacingX:CD_TRANSFER_COLUMNS], [dataOffset:sampleSpacingY:CD_TRANSFER_ROWS-dataOffset]);
  [X,Y] = meshgrid([1:CD_TRANSFER_COLUMNS], [1:CD_TRANSFER_ROWS]);
  img = interp2(X,Y,double(img),newX,newY,'linear');
