%%
% This function performs edge detection on the given image within the
% region of interest. The edge locations in mm are returned in the coordinate
% system defined by the files DICOM header. 
% NOTE: The function assumes isotropic pixels, as this is the assumption of
%       matlab's edge detector that it uses. If this is not the case the
%       function will issue a warning.
% 
% @param img The image slice on which we perform edge detection.
%
% @param dcmInfo Matlab's DICOM info structure which contains the location
%                of the image relative to the world coordinate system, and
%                the slice's pixel spacing which are required for computing
%                the edge locations in the 3D world coordinate system.
%
% @param minCoords Region of interest's minimal x and y coordinates
%                  [pixels].
%
% @param maxCoords Region of interest's maximal x and y coordinates
%                  [pixels].
%
% @param cannyStd Standard deviation used by the canny edge detector (for
%                 further information see matlab's 'edge' function).
%
% @param cannyLowThreshold Lower threshold value used by the canny edge detector (for
%                 further information see matlab's 'edge' function).
%
% @param cannyHighThreshold Upper threshold value used by the canny edge detector (for
%                 further information see matlab's 'edge' function).
%
% @return edgePoints A 3xn matrix containing the coordinates of the edge
%                    points [mm] in the world coordinate system.
%
% @author Ziv Yaniv (zivy@isis.georgetown.edu)
%

function edgePoints = getEdges3D(img, dcmInfo, minCoords, maxCoords, cannyStd, cannyLowThreshold, cannyHighThreshold)

subImage = img(round(minCoords(2)):round(maxCoords(2)),...
                     round(minCoords(1)):round(maxCoords(1)));

          %perform edge detection using the Canny detector, 
          %NOTE: the only reason we can use matlab's 'edge' function is
          %that the pixels are isotropic (see dicom header)
          %in general image processing in matlab does not have the notion
          %of pixel/voxel spacing, everything is assumed to be isotropic
edgeImage = edge(subImage, 'canny', [cannyLowThreshold, cannyHighThreshold], cannyStd);
edgeIndexes = find(edgeImage == 1);

     %no edges found, no need to continue processing
if isempty(edgeIndexes)
  edgePoints = [];
  return;
end

[y,x] = ind2sub(size(edgeImage), edgeIndexes);

     %draw the edges on the sub-image
% figureHandle = figure;
% outRed = double(subImage);
% maxVal = max(outRed(:));
% minVal = min(outRed(:));
% outRed = (outRed-minVal)/(maxVal-minVal);
% outGreen = outRed;
% outBlue = outRed;
% out = cat(3, outRed, outGreen, outBlue);
% red = [1;0;0];
% for i=1:size(x,1)
%   out(y(i),x(i),:) = red;
% end
% image(out);
% axis image;
% axis off;
% saveFigure(figureHandle, strcat(strtok(fileName, '.'),'_edges'));
% close(figureHandle);

         %get the edge locations in the original image
x = x + round(minCoords(1))-1;
y = y + round(minCoords(2))-1;

    %draw the edges on the original image
% figure;
% outRed = double(img);
% maxVal = max(outRed(:));
% minVal = min(outRed(:));
% outRed = (outRed-minVal)/(maxVal-minVal);
% outGreen = outRed;
% outBlue = outRed;
% out = cat(3, outRed, outGreen, outBlue);
% red = [1;0;0];
% for i=1:size(x,1)
%   out(y(i),x(i),:) = red;
% end
% image(out);
% axis image;
  
          %get the edge locations in homogenous coordinates, and change so
          %that the indexes start at zero and not one (the matlab
          %convention)
curPointSet = [x'-1;y'-1;zeros(1,size(x,1));ones(1,size(x,1))];
  
pixel2Volume = [dcmInfo.ImageOrientationPatient(1:3)*dcmInfo.PixelSpacing(1),...
                dcmInfo.ImageOrientationPatient(4:6)*dcmInfo.PixelSpacing(2),...
                cross(dcmInfo.ImageOrientationPatient(1:3),...
                      dcmInfo.ImageOrientationPatient(4:6)),...
                dcmInfo.ImagePositionPatient];
pixel2Volume = [pixel2Volume;[0 0 0 1]];                
                       
edgePoints = pixel2Volume*curPointSet;
edgePoints = edgePoints(1:3,:);
